@@ -39,7 +39,7 @@ TLD.prototype = {
       this.set(key1, key2, res);
     }
     return res;
-  }
+  },
 };
 
 dynamicforms = {
@@ -48,9 +48,11 @@ dynamicforms = {
     'template':          'dynamicforms/bootstrap/',
     'jquery_ui':         false,
     'edit_in_dialog':    true,
-    'bootstrap_version': 'v4'
+    'bootstrap_version': 'v4',
   },
 
+  filter_sequence: 0,
+
   /**
    * Presents the error in data exchange with the server in a user understandable way
    * @param xhr
@@ -1232,12 +1234,16 @@ dynamicforms = {
         table.find('tr').remove();
       }
       $("#loading-" + formID).show();
-      //TODO: Remember sequence number... if data that comes back has other than last sequence number than just ignore it #114
+      var curr_sequence = dynamicforms.filter_sequence;
       $.ajax({
                type:    'GET',
                headers: {'X-CSRFToken': dynamicforms.csrf_token, 'X-DF-RENDER-TYPE': 'table rows'},
                url:     link_next
              }).done(function (data) {
+        if (curr_sequence != dynamicforms.filter_sequence) {
+          console.log('Update table action was discarded as the filter has changed.');
+          return false;
+        }
 
         data                     = $(data).filter("tr");
         tbl_pagination.link_next = data[0].getAttribute('data-next');
@@ -1265,12 +1271,39 @@ dynamicforms = {
         dynamicforms.paginatorCheckGetNextPage(formID);
       }).fail(function (xhr, status, error) {
         $("#loading-" + formID).hide();
-        console.log('Pagination failed.', xhr, status, error);
-        // TODO: what if the server returns an error? Do we continue with pagination? (Task #100)
+        if (xhr.status == 403) {
+          var body = '<p>You are not authorized for this action.</p>';
+          dynamicforms.showModalDialog('Authorisation', body, [{title: 'OK', style: 'primary'}]);
+        } else if (xhr.status == 401 && dynamicforms.DYNAMICFORMS.login_url) {
+          var body = '<p>You will be redirected to login screen.</p>';
+          dynamicforms.showModalDialog('Authorisation', body, [{
+            title:      'Login',
+            style:      'primary',
+            callback:   'dynamicforms.redirectToLogin',
+            parameters: {path_next: link_next.replace(window.location.origin, '')},
+          }]);
+        } else {
+          var body = 'General server error.'
+          if (xhr.responseText) {
+            body = xhr.responseText;
+          }
+          dynamicforms.showModalDialog('Server error', body, [{title: 'OK', style: 'primary'}]);
+        }
       });
     }
   },
 
+  /*
+  * Callback function to redirect to login page
+   */
+  redirectToLogin: function redirectToLogin() {
+    var location = dynamicforms.DYNAMICFORMS.login_url;
+    if (this.path_next) {
+      location += '?next=' + this.path_next
+    }
+    window.location = location;
+  },
+
   /**
    * Goes through all tables that uses pagination and calls paginatorCheckGetNextPage function for them
    */
@@ -1309,10 +1342,12 @@ dynamicforms = {
    * @param returnDict: set to true if only filter data should be returned
    */
   filterData: function filterData(formID, returnDict) {
+    dynamicforms.filter_sequence++;
+
     if (returnDict == undefined)
       returnDict = false;
     var filter = {},
-        order = dynamicforms.df_tbl_pagination.get(formID, 'ordering');
+        order  = dynamicforms.df_tbl_pagination.get(formID, 'ordering');
 
     $("#list-" + formID).find(".dynamicforms-filterrow th").each(function (index) {
 
